#include <gnome.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

#include "internal.h"
#include "callbacks.h"

void initialize_app (GtkWidget * widget)
{
	InternalData *data;
	GtkListStore *list_store;
	GtkTreeViewColumn *column;

	/* Initialize an InternalData structure */
	data = (InternalData *) malloc (sizeof (InternalData));
	data->gconf_client = gconf_client_get_default ();
	data->gconf_change = NULL;
	data->gconf_key = GCONFPREF_BASE;
	data->view = NULL;
	data->view_top = NULL;
	data->description = lookup_widget (widget, "description");
	data->sidepane = lookup_widget (widget, "sidepane");

	/* Set the structure up in the app */
	g_object_set_data (G_OBJECT (widget), "data", (gpointer) data);

	/* Initialize the gconf_client */
	gconf_client_add_dir (data->gconf_client, data->gconf_key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

	/* Initialize the sidepane */
	list_store = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (data->sidepane), GTK_TREE_MODEL (list_store));
	column = gtk_tree_view_column_new_with_attributes ("Another Column", gtk_cell_renderer_pixbuf_new(), "pixbuf", 0, NULL);
	gtk_tree_view_insert_column (GTK_TREE_VIEW (data->sidepane), GTK_TREE_VIEW_COLUMN (column), 0);

	column = gtk_tree_view_column_new_with_attributes("A Column", gtk_cell_renderer_text_new (), "text", 1, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (data->sidepane), GTK_TREE_VIEW_COLUMN (column));

	refresh_sidepane (data);
}

void finalize_app (GtkWidget * widget)
{
	InternalData *data;

	data = get_data_from_widget (widget);
	g_object_unref (G_OBJECT (data->gconf_client));

	gtk_main_quit ();
}

void refresh_sidepane (InternalData * data)
{
	GtkListStore *list_store;
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	GSList *list;

	list_store = GTK_LIST_STORE (gtk_tree_view_get_model (data->sidepane));
	list = internal_tidy_list (gconf_client_all_dirs(data->gconf_client, data->gconf_key, NULL));

	for (; list != NULL; list = g_slist_next (list))
	{
		gtk_list_store_append (list_store, &iter);
		gtk_list_store_set (list_store, &iter, 0, internal_get_image(g_path_get_basename(list->data)), 1, internal_str_beautify (list->data), 2, list->data, -1);
	}

	selection = gtk_tree_view_get_selection (data->sidepane);
	g_signal_connect (selection, "changed", on_sidepane_selection_changed, NULL);
	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list_store), &iter);
	gtk_tree_selection_select_iter (selection, &iter);

}

void refresh_view (InternalData * data)
{
	char *key, *oldkey;
	GtkContainer *container;
	GtkTreeIter iter;
	GtkTreeModel *model;

	gtk_tree_selection_get_selected (gtk_tree_view_get_selection(data->sidepane), &model, &iter);
	gtk_tree_model_get (model, &iter, 2, &key, -1);

	container =GTK_CONTAINER (lookup_widget (data->sidepane, "vpanedview"));
	if(data->view_top!=NULL)
	{
			gtk_container_remove (container, data->view_top);
	}
	data->view_top = internal_create_view (key, data);
	gtk_paned_pack1 (GTK_PANED (container), data->view_top, TRUE, FALSE);
	gtk_widget_show (data->view_top);

	g_free (key);
}

InternalData *get_data_from_widget (GtkWidget * widget)
{
	return g_object_get_data (G_OBJECT(lookup_widget(GTK_WIDGET (widget), "app")), "data");
}

/*
 * The 'more' internal functions
 */

GtkWidget *internal_create_view (char *key, InternalData * data)
{
	GtkWidget *widget, *view, *label;
	GSList *dirlist, entrylist;
	char *caption;
	
	dirlist = internal_tidy_list (gconf_client_all_dirs(data->gconf_client, key, NULL));
	
	if (dirlist == NULL)
	{
		return internal_create_widgets_from_list(gconf_client_all_entries(data->gconf_client, key, NULL));
	}
	else
	{
		widget = gtk_notebook_new();
		for (;dirlist!=NULL;dirlist=g_slist_next(dirlist))
		{
			caption = internal_str_beautify(dirlist->data);
			label = gtk_label_new(caption);
			view = internal_create_view(dirlist->data, data);
			//internal_create_widgets_from_list(gconf_client_all_entries(data->gconf_client, key, NULL));
			gtk_notebook_append_page(widget, view, label);
			gtk_widget_show(label);
			gtk_widget_show(view);
			g_free(caption);
		}
	}
	
	return widget;
}

/* Improve this WHOLE function */
GtkWidget *internal_create_widgets_from_list (GSList * list)
{
	GtkWidget *viewport, *table, *widget1, *widget2;
	GConfEntry *entry;
	char *name;
	int rows;

	rows = 0;
	table = gtk_table_new (rows, 2, FALSE);

	for (; list != NULL; list = g_slist_next (list))
	{
		rows++;
		entry = list->data;
		name = internal_str_beautify (gconf_entry_get_key (entry));

		gtk_table_resize (GTK_TABLE (table), rows, 2);

		if(entry->value==NULL)
			break;

		switch (entry->value->type)
		{
		case GCONF_VALUE_STRING:
			widget1 = gtk_label_new (name);
			widget2 = gtk_entry_new ();
			if(entry->value==NULL)
			{
				gtk_entry_set_text (GTK_ENTRY (widget2), "<no value>");
			} else {
				gtk_entry_set_text (GTK_ENTRY (widget2), gconf_value_get_string (entry->value));
			}	
			gtk_table_attach (GTK_TABLE (table), widget1, 0, 1, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_table_attach (GTK_TABLE (table), widget2, 1, 2, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_widget_show (widget1);
			gtk_widget_show (widget2);
			break;
		case GCONF_VALUE_INT:
			widget1 = gtk_label_new (name);
			widget2 = gtk_spin_button_new_with_range (-100, 100, 1);
			if(entry->value==NULL)
			{
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget2), -1);
			} else {
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget2), gconf_value_get_int (entry->value));
			}
			gtk_table_attach (GTK_TABLE (table), widget1, 0, 1, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_table_attach (GTK_TABLE (table), widget2, 1, 2, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_widget_show (widget1);
			gtk_widget_show (widget2);
			break;
		case GCONF_VALUE_FLOAT:
			widget1 = gtk_label_new (name);
			widget2 = gtk_spin_button_new_with_range (-100, 100, 0.01);
			if(entry->value==NULL)
			{
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget2), -1);
			} else {
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget2), gconf_value_get_float(entry->value));
			}
			gtk_table_attach (GTK_TABLE (table), widget1, 0, 1, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_table_attach (GTK_TABLE (table), widget2, 1, 2, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_widget_show (widget1);
			gtk_widget_show (widget2);
			break;
		case GCONF_VALUE_BOOL:
			widget1 = gtk_check_button_new_with_label (name);
			if(entry->value==NULL)
			{
				gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(widget1), 1);
			} else {
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget1), gconf_value_get_bool(entry->value));
			}
			gtk_table_attach (GTK_TABLE (table), widget1, 0, 2, rows, rows + 1, GTK_SHRINK, GTK_SHRINK, 0, 1);
			gtk_widget_show (widget1);
			break;
		}
		
		g_free (name);
	}
	
	viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_add_with_viewport(viewport, table);
	gtk_widget_show(table);
	
	return viewport;
}

/* a bit useless, can be replaced with g_utf8_collate (or the ascii equivalent) */
int internal_compare_chars (gconstpointer a, gconstpointer b)
{
	return g_utf8_collate (g_path_get_basename (a), g_path_get_basename (b));
}

/* Needs to be redone to be more gnome correct.
	Should probably be tied in with the functions in the support.h/.c files */
GdkPixbuf *internal_get_image (char *data)
{
	char *cur, *filename, *temp;
	GdkPixbuf *image;

	cur = g_get_current_dir ();
	temp = g_strconcat (data, ".png", NULL);
	filename = g_build_filename (cur, GCONFPREF_IMGDIR, temp, NULL);
	g_free (temp);

	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
	{
		g_free (filename);
		filename = g_build_filename (cur, GCONFPREF_IMGDIR, GCONFPREF_IMGDEF, NULL);
	}
	g_free (cur);

	/* Now that we have the filename, get the image */
	image = gdk_pixbuf_new_from_file (filename, NULL);

	g_free (filename);

	return image;
}

/* How can this function be improved? */
char *internal_str_beautify (char *str)
{
	char *return_str;
	int i;

	return_str = g_path_get_basename (str);

	return_str[0] = g_unichar_toupper (return_str[0]);
	for (i = 0; return_str[i] != NULL; i++)
	{
		if (return_str[i] == '_' || return_str[i] == '-')
			return_str[i] = ' ';
		if (i > 0 && return_str[i - 1] == ' ')
			return_str[i] = g_unichar_toupper (return_str[i]);
	}

	return return_str;
}


/* Currently this function just sorts, maybe more later... */
GSList *internal_tidy_list (GSList * list)
{
	GSList *return_list;

	return_list = g_slist_sort (list, internal_compare_chars);

	return return_list;
}
